﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Windows;
using Forum.DataAccess;
using System.Threading;
using Forum.Model;
using System.Windows.Input;
using Forum.Components;
using System.Windows.Data;

namespace Forum.ViewModel
{
    public class MainWindowViewModel : ViewModelBase, IWeakEventListener
    {

        private ThreadPostViewModelCollection _threads;
        private ICommand _refreshCommand;
        private ICommand _closeCommand;
        private ICommand _newCommand;
        private ICommand _saveCommand;

        public MainWindowViewModel()
        {
            _refreshCommand = new RelayCommand(p => RefreshAsync(), p => !this.IsLoading);
            _closeCommand = new RelayCommand(p => OnRequestClose(EventArgs.Empty));
            _newCommand = new RelayCommand(p => NewThreadPost());
            _saveCommand = new RelayCommand(p => this.SaveThreadPost(), p => this.CanSave);
        }

        public void NewThreadPost()
        {
            ThreadPost tp = new ThreadPost();
            tp.Posts = new List<Post> { new Post() };

            ThreadPostViewModel t = new NewThreadPostViewModel(tp);
            this.Threads.Add(t);

            ICollectionView view = CollectionViewSource.GetDefaultView(this.Threads);
            view.MoveCurrentTo(t);
        }

        public void SaveThreadPost()
        {
            ICollectionView view = CollectionViewSource.GetDefaultView(this.Threads);

            NewThreadPostViewModel t = view.CurrentItem as NewThreadPostViewModel;
            if (t != null)
            {
                ThreadPostViewModel tp = new ThreadPostViewModel(t.InternalThreadPost);
                this.Threads.Remove(t);
                this.Threads.Add(tp);

                view.MoveCurrentTo(tp);
            }
        }

        public void RefreshAsync()
        {
            // Notifico l'esecuzione asincrona
            this.IsLoading = true;
            // Creo un'operazione che rappresenta il refresh
            AsyncOperation refreshOperation = AsyncOperationManager.CreateOperation(null);

            // Invoco in asincrono
            ThreadPool.QueueUserWorkItem(delegate
            {
                try
                {
                    IEnumerable<ThreadPost> threads = ForumManager.GetAllThreads();

                    // Invoco sul thread UI la funzione
                    refreshOperation.PostOperationCompleted(OnRefreshAsyncCompleted, threads);
                }
                catch (Exception)
                {
                    refreshOperation.PostOperationCompleted(OnRefreshAsyncFailed, null);
                }
            }, refreshOperation);
        }

        private bool CanSave
        {
            get
            {
                ICollectionView view = CollectionViewSource.GetDefaultView(this.Threads);

                NewThreadPostViewModel t = view.CurrentItem as NewThreadPostViewModel;
                if (t != null)
                {
                    return t.CanSave;
                }

                return false;
            }
        }

        private void OnRefreshAsyncFailed(object arg)
        {
            this.IsLoading = false;
            CommandManager.InvalidateRequerySuggested();
        }

        private void OnRefreshAsyncCompleted(object arg)
        {
            this.IsLoading = false;
            CommandManager.InvalidateRequerySuggested();

            // Preparo il ViewModel per i thread
            IEnumerable<ThreadPost> threads = (IEnumerable<ThreadPost>)arg;
            this.Threads.Clear();
            foreach (ThreadPost tp in threads)
                this.Threads.Add(new ThreadPostViewModel(tp));

            ICollectionView view = CollectionViewSource.GetDefaultView(this.Threads);
            view.MoveCurrentToFirst();
        }

        public ICommand RefreshCommand
        {
            get { return _refreshCommand; }
        }

        public ICommand CloseCommand
        {
            get { return _closeCommand; }
        }

        public ICommand NewCommand
        {
            get { return _newCommand; }
        }

        public ICommand SaveCommand
        {
            get { return _saveCommand; }
        }

        public ThreadPostViewModelCollection Threads
        {
            get
            {
                if (_threads == null)
                {
                    _threads = new ThreadPostViewModelCollection();
                    CollectionChangedEventManager.AddListener(_threads, this);
                    CurrentChangedEventManager.AddListener(CollectionViewSource.GetDefaultView(_threads), this);
                }

                return _threads;
            }
        }

        private bool _isLoading;

        public bool IsLoading
        {
            get { return _isLoading; }
            private set
            {
                _isLoading = value;
                OnPropertyChanged("IsLoading");
            }
        }

        public int TotalReadThreads
        {
            get { return this.Threads.Count(t => t.IsRead); }
        }

        public event EventHandler RequestClose;

        protected virtual void OnRequestClose(EventArgs e)
        {
            if (this.RequestClose != null)
                this.RequestClose(this, e);
        }

        #region IWeakEventListener Members

        bool IWeakEventListener.ReceiveWeakEvent(Type managerType, object sender, EventArgs e)
        {
            // Evento scatenato da INotifyCollectionChanged
            if (sender == this.Threads)
            {
                NotifyCollectionChangedEventArgs ce = (NotifyCollectionChangedEventArgs)e;

                switch (ce.Action)
                {
                    // Mi metto in ascolto sul cambio di proprietà IsRead
                    case NotifyCollectionChangedAction.Add:
                        foreach (object item in ce.NewItems)
                            PropertyChangedEventManager.AddListener((INotifyPropertyChanged)item, this, "IsRead");
                        break;
                    // Non occorre più che mi metta in ascolto sul cambio di proprietà IsRead
                    case NotifyCollectionChangedAction.Remove:
                        foreach (object item in ce.OldItems)
                            PropertyChangedEventManager.RemoveListener((INotifyPropertyChanged)item, this, "IsRead");
                        break;
                }

                // L'evento è stato gestito
                return true;
            }
            else if (sender is ICollectionView)
            {
                ThreadPostViewModel t = ((ICollectionView)sender).CurrentItem as ThreadPostViewModel;
                if (t != null)
                    t.IsRead = true;

                return true;
            }
            // Evento PropertyChanged invocato sul Thread
            else if (sender is ThreadPostViewModel)
            {
                // Notifico che la proprietà è cambiata
                // così verrà re interrogata e calcolata
                OnPropertyChanged("TotalReadThreads");

                return true;
            }

            return false;
        }

        #endregion
    }

}
